#include "Adafruit_GFX.h"
#include "Adafruit_HT1632.h"
#include "Maxbotix.h"

#define LENGTH(array) (sizeof(array) / sizeof(array[0]))

const byte DATA_PIN = 2;
const byte WR_PIN = 3;
const byte CS_PIN = 4;
const byte RANGE_PIN = A5;
const byte SAMPLE_SIZE = 32;

const byte CALIBRATION_READS = 6;
const byte DELAY = 1;

class FilterBuLp2 {
  public:
    FilterBuLp2() {
      v[0] = 0.0;
      v[1] = 0.0;
    }
    
  private:
    float v[3];
  public:
    float step(float x) //class II
    {
      v[0] = v[1];
      v[1] = v[2];
      v[2] = (2.995458220809249505e-2 * x) + (-0.57406191508395476575 * v[0]) + (1.45424358625158478553 * v[1]);
      return (v[0] + v[2]) + 2 * v[1];
    }
};

Adafruit_HT1632LEDMatrix matrix(DATA_PIN, WR_PIN, CS_PIN);
Maxbotix rangeSensor(RANGE_PIN, Maxbotix::AN, Maxbotix::LV, Maxbotix::BEST, SAMPLE_SIZE);
FilterBuLp2 filter;

int16_t buffer[24];
byte bufferIndex = 0;

void setup() {
  memset(buffer, 0, LENGTH(buffer));
  for (byte i = 0; i < CALIBRATION_READS; ++i) {
    // Calibrate the filter by doing some sample reads
    filter.step(rangeSensor.getRange());
  }
  
  matrix.begin(ADA_HT1632_COMMON_16NMOS);
  matrix.drawPixel(0, 0, 1);
  matrix.writeScreen();
  delay(1000);
}

void loop() {
  matrix.clearScreen();
  for (int16_t i = 0; i < LENGTH(buffer); ++i) {
    int16_t index = (bufferIndex + i) % LENGTH(buffer);
    matrix.drawFastVLine(i, (int16_t)15, buffer[index], 1);
  }
  matrix.writeScreen();

  float range = filter.step(rangeSensor.getRange());
  buffer[bufferIndex] = map(range, 100, 200, 0, 15);
  bufferIndex = (bufferIndex + 1) % LENGTH(buffer);

  delay(DELAY);
}

